# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Standard library imports.
import json
from StringIO import StringIO
from zipfile import ZipFile, ZIP_DEFLATED

# Python third-party library imports.
import mock

# Django imports.
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.http import Http404
from django.test import RequestFactory
from django.test import TestCase
from django.utils.http import urlquote

# Application imports
from conntrackt.models import Project, Location, Entity, Interface, Communication

from conntrackt.views import IndexView, SearchView, APISearchView
from conntrackt.views import entity_iptables, project_iptables, project_diagram

from conntrackt.views import ProjectView, ProjectCreateView, ProjectUpdateView, ProjectDeleteView
from conntrackt.views import LocationCreateView, LocationUpdateView, LocationDeleteView
from conntrackt.views import EntityView, EntityCreateView, EntityUpdateView, EntityDeleteView
from conntrackt.views import InterfaceCreateView, InterfaceUpdateView, InterfaceDeleteView
from conntrackt.views import CommunicationCreateView, CommunicationUpdateView, CommunicationDeleteView

# Test imports.
from .forms import FormWithWidgetCSSClassFormMixin, FormWithPlaceholderFormMixin
from .helpers import PermissionTestMixin, RenderTestMixin, create_get_request, generate_get_response, FakeMessages
from .views import RedirectToNextMixinView
from .factories import setup_test_data


class IndexViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    sufficient_permissions = ("view",)
    view_class = IndexView

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context_no_projects(self):
        """
        Verifies that the context is properly set-up when the view is called and
        no projects are available.
        """

        Project.objects.all().delete()

        # Get the view.
        view = IndexView.as_view()

        # Get the response.
        response = generate_get_response(view)

        # Validate the response.
        self.assertQuerysetEqual(response.context_data["projects"], [])

    def test_context_no_locations(self):
        """
        Verifies that the context is properly set-up when the view is called and
        no locations are available.
        """

        Location.objects.all().delete()

        # Get the view.
        view = IndexView.as_view()

        # Get the response.
        response = generate_get_response(view)

        # Validate the response.
        self.assertQuerysetEqual(response.context_data["locations"], [])

    def test_context_projects(self):
        """
        Verifies that the context is properly set-up when the view is called and
        there's multiple projects available.
        """

        # Get the view.
        view = IndexView.as_view()

        # Get the response.
        response = generate_get_response(view)

        self.assertQuerysetEqual(response.context_data["projects"], ["<Project: Test Project 1>", "<Project: Test Project 2>"])

    def test_locations_available(self):
        """
        Verifies that the context is properly set-up when the view is called and
        there's multiple locationsg available.
        """

        # Get the view.
        view = IndexView.as_view()

        # Get the response.
        response = generate_get_response(view)

        # Validate the response.
        self.assertQuerysetEqual(response.context_data["locations"], ["<Location: Test Location 1>", "<Location: Test Location 2>"])


class ProjectViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    sufficient_permissions = ("view",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}
    view_class = ProjectView

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called.
        """

        # Get the view.
        view = ProjectView.as_view()

        # Get the response.
        response = generate_get_response(view, pk=1)

        # Fetch context data from response.
        location, entities = response.context_data["location_entities"][0]

        # Set-up expected context data values.
        expected_entities = ["<Entity: Test Entity 1 (Test Project 1 - Test Location 1)>",
                             "<Entity: Test Entity 2 (Test Project 1 - Test Location 1)>"]

        # Validate context data.
        self.assertEqual(location.name, "Test Location 1")
        self.assertQuerysetEqual(entities, expected_entities)

        # Fetch context data from response.
        location, entities = response.context_data["location_entities"][1]

        # Set-up expected context data values.
        expected_entities = ["<Entity: Test Entity 3 (Test Project 1 - Test Location 2)>",
                             "<Entity: Test Subnet 4 (Test Project 1 - Test Location 2)>"]

        # Validate context data.
        self.assertEqual(location.name, "Test Location 2")
        self.assertQuerysetEqual(entities, expected_entities)

        # Validate context data.
        self.assertEqual(str(response.context_data["project"]), "Test Project 1")

        # Validate context data is present.
        self.assertIn("communications", response.context_data.keys())


class EntityViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = EntityView
    sufficient_permissions = ("view",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Tests if the form comes pre-populated with proper content.
        """

        # Get the view.
        view = EntityView.as_view()

        # Get the response.
        response = generate_get_response(view, pk=1)

        # Set-up expected context data.
        expected_entity = Entity.objects.get(pk=1)

        expected_incoming_communications = ["<Communication: Test Entity 2 -> Test Entity 1 (TCP:22)>",
                                            "<Communication: Test Entity 2 -> Test Entity 1 (ICMP:8)>",
                                            "<Communication: Test Entity 3 -> Test Entity 1 (TCP:3306)>",
                                            "<Communication: Test Subnet 4 -> Test Entity 1 (TCP:22)>"]

        expected_outgoing_communications = ["<Communication: Test Entity 1 -> Test Entity 3 (UDP:53)>",
                                            "<Communication: Test Entity 1 -> Test Entity 2 (UDP:123)>"]

        expected_interfaces = ["<Interface: Test Entity 1 (192.168.1.1)>"]

        # Validate the response.
        self.assertQuerysetEqual(response.context_data["interfaces"], expected_interfaces)
        self.assertQuerysetEqual(response.context_data["incoming_communications"], expected_incoming_communications)
        self.assertQuerysetEqual(response.context_data["outgoing_communications"], expected_outgoing_communications)
        self.assertEqual(response.context_data["entity"], expected_entity)
        self.assertTrue("entity_iptables" in response.context_data)


class EntityIptablesTest(PermissionTestMixin, TestCase):

    view_function = staticmethod(entity_iptables)
    sufficient_permissions = ("view",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_invalid_entity(self):
        """
        Tests if a 404 is returned if no entity was found (invalid ID).
        """

        # Set-up a request.
        request = create_get_request()

        # Get the view.
        view = entity_iptables

        # Validate the response.
        self.assertRaises(Http404, view, request, pk=200)

    def test_content_type(self):
        """
        Test if correct content type is being returned by the response.
        """

        # Get the view.
        view = entity_iptables

        # Get the response.
        response = generate_get_response(view, pk=1)

        self.assertEqual(response['Content-Type'], "text/plain")

    def test_content_disposition(self):
        """
        Test if the correct content disposition has been set.
        """

        # Get the view.
        view = entity_iptables

        # Get the response.
        response = generate_get_response(view, pk=1)

        self.assertEqual(response['Content-Disposition'], "attachment; filename=test_entity_1-iptables.conf")

    def test_content(self):
        """
        Tests content produced by the view.
        """

        # Get the view.
        view = entity_iptables

        # Get the response.
        response = generate_get_response(view, pk=1)

        self.assertContains(response, ":INPUT")
        self.assertContains(response, ":OUTPUT")
        self.assertContains(response, ":FORWARD")


class ProjectIptablesTest(PermissionTestMixin, TestCase):

    view_function = staticmethod(project_iptables)
    sufficient_permissions = ("view",)
    permission_test_view_kwargs = {"project_id": 1}
    render_test_view_kwargs = {"project_id": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_invalid_project(self):
        """
        Tests if a 404 is returned if no project was found (invalid ID).
        """

        # Set-up a request.
        request = create_get_request()

        # Get the view.
        view = project_iptables

        # Request iptables for whole project.
        self.assertRaises(Http404, view, request, 200)
        # Request iptables for project location
        self.assertRaises(Http404, view, request, 200, 1)

    def test_invalid_location(self):
        """
        Tests if a 404 is returned if no location was found (invalid ID).
        """

        # Set-up a request.
        request = create_get_request()

        # Get the view.
        view = project_iptables

        # Request iptables for project location
        self.assertRaises(Http404, view, request, 1, 200)

    def test_content_type(self):
        """
        Test if correct content type is being returned by the response.
        """

        # Get the view.
        view = project_iptables

        # Get the response.
        response = generate_get_response(view, None, 1)

        # Validate the response.
        self.assertEqual(response['Content-Type'], "application/zip")

    def test_content_disposition(self):
        """
        Test if the correct content disposition has been set.
        """

        # Get the view.
        view = project_iptables

        # Get the response.
        response = generate_get_response(view, None, 1)
        self.assertEqual(response['Content-Disposition'], 'attachment; filename="test_project_1-iptables.zip"')

        response = generate_get_response(view, None, 1, 1)
        self.assertEqual(response['Content-Disposition'], 'attachment; filename="test_project_1-test_location_1-iptables.zip"')

    def test_content_project(self):
        """
        Verifies that the content is properly generated when the view is called
        for an entire project.
        """

        # Get the view.
        view = project_iptables

        # Get the response.
        response = generate_get_response(project_iptables, None, 1)

        buff = StringIO(response.content)

        zipped_iptables = ZipFile(buff, "r", ZIP_DEFLATED)
        expected_zip_files = ["test_entity_1-iptables.conf",
                              "test_entity_2-iptables.conf",
                              "test_entity_3-iptables.conf",
                              "test_subnet_4-iptables.conf"]

        self.assertEqual(len(zipped_iptables.namelist()), 4)
        self.assertEqual(zipped_iptables.namelist(), expected_zip_files)

        for filename in expected_zip_files:
            iptables_file = zipped_iptables.read(filename)
            self.assertIn(":INPUT", iptables_file)
            self.assertIn(":OUTPUT", iptables_file)
            self.assertIn(":FORWARD", iptables_file)

        zipped_iptables.close()

    def test_content_location(self):
        """
        Verifies that the content is properly generated when the view is called
        for an entire project.
        """

        # Get the view.
        view = project_iptables

        # Get the response.
        response = generate_get_response(project_iptables, None, 1, 1)

        buff = StringIO(response.content)

        zipped_iptables = ZipFile(buff, "r", ZIP_DEFLATED)
        expected_zip_files = ["test_entity_1-iptables.conf",
                              "test_entity_2-iptables.conf"]

        self.assertEqual(len(zipped_iptables.namelist()), 2)
        self.assertEqual(zipped_iptables.namelist(), expected_zip_files)

        for filename in expected_zip_files:
            iptables_file = zipped_iptables.read(filename)
            self.assertIn(":INPUT", iptables_file)
            self.assertIn(":OUTPUT", iptables_file)
            self.assertIn(":FORWARD", iptables_file)

        zipped_iptables.close()


class ProjectCreateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = ProjectCreateView
    sufficient_permissions = ("add_project",)


class ProjectUpdateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = ProjectUpdateView
    sufficient_permissions = ("change_project",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific project.
        """

        # Get the view.
        view = ProjectUpdateView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["project"].name, "Test Project 1")
        self.assertEqual(response.context_data["headline"], "Update project Test Project 1")


class ProjectDeleteViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = ProjectDeleteView
    sufficient_permissions = ("delete_project",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific project.
        """

        # Get the expected project.
        project = Project.objects.get(pk=1)

        # Get the view.
        view = ProjectDeleteView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["project"], project)
        self.assertEqual(response.context_data["headline"], "Delete project Test Project 1")

    def test_message(self):
        """
        Tests if the message gets added when the project is deleted.
        """

        # Get the view.
        view = ProjectDeleteView.as_view()

        # Generate the request.
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertIn("Project Test Project 1 has been removed.", request._messages.messages)


class LocationCreateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = LocationCreateView
    sufficient_permissions = ("add_location",)


class LocationUpdateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = LocationUpdateView
    sufficient_permissions = ("change_location",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific location.
        """

        # Get the view.
        view = LocationUpdateView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["location"].name, "Test Location 1")
        self.assertEqual(response.context_data["headline"], "Update location Test Location 1")


class LocationDeleteViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = LocationDeleteView
    sufficient_permissions = ("delete_location",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}    

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific location.
        """

        # Get the expected location.
        location = Location.objects.get(pk=1)

        # Get the view.
        view = LocationDeleteView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["location"], location)
        self.assertEqual(response.context_data["headline"], "Delete location Test Location 1")

    def test_message(self):
        """
        Tests if the message gets added when the location is deleted.
        """

        # Get the view.
        view = LocationDeleteView.as_view()

        # Generate the request.
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertIn("Location Test Location 1 has been removed.", request._messages.messages)


class EntityCreateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = EntityCreateView
    sufficient_permissions = ("add_entity",)

    def setUp(self):
        """
        Sets-up some data necessary for testing.
        """

        # Set-up some data for testing.
        Project.objects.create(name="Test Project 1", description="This is test project 1.")
        Project.objects.create(name="Test Project 2", description="This is test project 2.")
        Location.objects.create(name="Test Location 1", description="This is test location 1.")
        Location.objects.create(name="Test Location 2", description="This is test location 2.")

    def test_form_project_limit(self):
        """
        Tests if the queryset is properly limitted to specific project if GET
        parameters is passed.
        """

        # Set-up the view.
        view = EntityCreateView()
        view.request = RequestFactory().get("/fake-path?project=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        self.assertQuerysetEqual(form.fields["project"].queryset, ["<Project: Test Project 1>"])

    def test_form_location_limit(self):
        """
        Tests if the queryset is properly limitted to specific location if GET
        parameters is passed.
        """

        # Set-up the view.
        view = EntityCreateView()
        view.request = RequestFactory().get("/fake-path?location=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        self.assertQuerysetEqual(form.fields["location"].queryset, ["<Location: Test Location 1>"])

    def test_initial_project(self):
        """
        Tests if the choice field for project is defaulted to project passed as
        part of GET parameters.
        """

        view = EntityCreateView()
        view.request = RequestFactory().get("/fake-path?project=1")
        view.object = None

        initial = view.get_initial()

        self.assertDictContainsSubset({"project": "1"}, initial)

    def test_initial_location(self):
        """
        Tests if the choice field for location is defaulted to location passed
        as part of GET parameters.
        """

        view = EntityCreateView()
        view.request = RequestFactory().get("/fake-path?location=1")
        view.object = None

        initial = view.get_initial()

        self.assertDictContainsSubset({"location": "1"}, initial)


class EntityDeleteViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = EntityDeleteView
    sufficient_permissions = ("delete_entity",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific entity.
        """

        # Get the expected entity.
        entity = Entity.objects.get(pk=1)

        # Get the view.
        view = EntityDeleteView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["entity"], entity)
        self.assertEqual(response.context_data["headline"], "Delete entity Test Entity 1")

    def test_message(self):
        """
        Tests if the message gets added when the entity is deleted.
        """

        # Get the view.
        view = EntityDeleteView.as_view()

        # Generate the request.
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertIn("Entity Test Entity 1 has been removed.", request._messages.messages)

    def test_success_url(self):
        """
        Validate that the success URL is set properly after delete.
        """

        # Get the view.
        view = EntityDeleteView.as_view()

        # Generate the request
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("project", args=(1,)))


class EntityUpdateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = EntityUpdateView
    sufficient_permissions = ("change_entity",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific entity.
        """

        # Get the view.
        view = EntityUpdateView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["entity"].name, "Test Entity 1")
        self.assertEqual(response.context_data["headline"], "Update entity Test Entity 1")


class InterfaceCreateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = InterfaceCreateView
    sufficient_permissions = ("add_interface",)

    def setUp(self):
        """
        Sets-up some data necessary for testing.
        """

        # Set-up some data for testing.
        project = Project.objects.create(name="Test Project", description="This is test project.")
        location = Location.objects.create(name="Test Location", description="This is test location.")
        Entity.objects.create(name="Test Entity 1", description="This is test entity 1.", project=project, location=location)
        Entity.objects.create(name="Test Entity 2", description="This is test entity 2.", project=project, location=location)

    def test_form_entity_limit(self):
        """
        Tests if the queryset is properly limitted to specific entity if GET
        parameter is passed.
        """

        # Set-up the view.
        view = InterfaceCreateView()
        view.request = RequestFactory().get("/fake-path?entity=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        self.assertQuerysetEqual(form.fields["entity"].queryset, ["<Entity: Test Entity 1 (Test Project - Test Location)>"])

    def test_initial_project(self):
        """
        Tests if the choice field for entity is defaulted to entity passed as
        part of GET parameters.
        """

        view = InterfaceCreateView()
        view.request = RequestFactory().get("/fake-path?entity=1")
        view.object = None

        initial = view.get_initial()

        self.assertDictContainsSubset({"entity": "1"}, initial)

    def test_success_url(self):
        """
        Validate that the success URL is set properly after interface is
        created.
        """

        # Get the view.
        view = InterfaceCreateView.as_view()

        # Generate the request.
        post_data = {"name": "eth0", "description": "Main interface.",
                     "entity": "1", "address": "192.168.1.1",
                     "netmask": "255.255.255.255"}
        request = RequestFactory().post("/fake-path/", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(1,)))
        self.assertEqual(response.status_code, 302)


class InterfaceUpdateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = InterfaceUpdateView
    sufficient_permissions = ("change_interface",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific entity.
        """

        # Get the view.
        view = InterfaceUpdateView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        # Set-up expected interface.
        interface = Interface.objects.get(pk=1)

        self.assertEqual(response.context_data["interface"], interface)
        self.assertEqual(response.context_data["headline"], "Update interface eth0")

    def test_form_entity_limit(self):
        """
        Tests if the queryset is properly limitted to specific project's
        entities.
        """

        # Set-up the view.
        view = InterfaceUpdateView()
        view.request = RequestFactory().get("/fake-path/1")
        view.object = Interface.objects.get(pk=1)

        # Get the form.
        form = view.get_form(view.get_form_class())

        expected_entities = ["<Entity: Test Entity 1 (Test Project 1 - Test Location 1)>",
                             "<Entity: Test Entity 2 (Test Project 1 - Test Location 1)>",
                             "<Entity: Test Entity 3 (Test Project 1 - Test Location 2)>",
                             "<Entity: Test Subnet 4 (Test Project 1 - Test Location 2)>"]

        self.assertQuerysetEqual(form.fields["entity"].queryset, expected_entities)

    def test_success_url(self):
        """
        Validate that the success URL is set properly after update.
        """

        # Get the view.
        view = InterfaceUpdateView.as_view()

        # Get the interface object.
        interface = Interface.objects.get(pk=1)

        # Generate the request.
        post_data = {"name": interface.name, "description": interface.name,
                     "entity": "1", "address": "192.168.1.1",
                     "netmask": "255.255.255.255"}
        request = RequestFactory().post("/fake-path/", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(1,)))
        self.assertEqual(response.status_code, 302)


class InterfaceDeleteViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = InterfaceDeleteView
    sufficient_permissions = ("delete_interface",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific interface.
        """

        # Get the expected entity.
        interface = Interface.objects.get(pk=1)

        # Get the view.
        view = InterfaceDeleteView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["interface"], interface)
        self.assertEqual(response.context_data["headline"], "Delete interface eth0")

    def test_message(self):
        """
        Tests if the message gets added when the interface is deleted.
        """

        # Get the view.
        view = InterfaceDeleteView.as_view()

        # Generate the request.
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertIn("Interface eth0 has been removed.", request._messages.messages)

    def test_success_url(self):
        """
        Validate that the success URL is set properly after delete.
        """

        # Get the view.
        view = InterfaceDeleteView.as_view()

        # Generate the request
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(1,)))


class CommunicationCreateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = CommunicationCreateView
    sufficient_permissions = ("add_communication",)

    def setUp(self):
        """
        Sets-up some data necessary for testing.
        """

        # Set-up some data for testing.
        project1 = Project.objects.create(name="Test Project 1", description="This is test project 1.")
        project2 = Project.objects.create(name="Test Project 2", description="This is test project 2.")
        location = Location.objects.create(name="Test Location", description="This is test location.")
        entity1 = Entity.objects.create(name="Test Entity 1", description="This is test entity 1.", project=project1, location=location)
        entity2 = Entity.objects.create(name="Test Entity 2", description="This is test entity 2.", project=project1, location=location)
        entity3 = Entity.objects.create(name="Test Entity 3", description="This is test entity 3.", project=project2, location=location)
        Interface.objects.create(name="eth0", description="Main interface", entity=entity1, address="192.168.1.1", netmask="255.255.255.255")
        Interface.objects.create(name="eth0", description="Main interface", entity=entity2, address="192.168.1.2", netmask="255.255.255.255")
        Interface.objects.create(name="eth0", description="Main interface", entity=entity3, address="192.168.1.3", netmask="255.255.255.255")

    def test_interface_limit_from_entity(self):
        """
        Tests if the queryset is properly limitted if GET parameter is passed.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?from_entity=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        # Set-up expected interfaces.
        expected_interfaces = ["<Interface: Test Entity 1 (192.168.1.1)>",
                               "<Interface: Test Entity 2 (192.168.1.2)>"]

        self.assertQuerysetEqual(form.fields["source"].queryset, expected_interfaces)
        self.assertQuerysetEqual(form.fields["destination"].queryset, expected_interfaces)

    def test_interface_limit_to_entity(self):
        """
        Tests if the queryset is properly limitted if GET parameter is passed.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?to_entity=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        # Set-up expected interfaces.
        expected_interfaces = ["<Interface: Test Entity 1 (192.168.1.1)>",
                               "<Interface: Test Entity 2 (192.168.1.2)>"]

        self.assertQuerysetEqual(form.fields["source"].queryset, expected_interfaces)
        self.assertQuerysetEqual(form.fields["destination"].queryset, expected_interfaces)

    def test_interface_limit_project(self):
        """
        Tests if the queryset is properly limitted if GET parameter is passed.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?project=1")
        view.object = None

        # Get the form.
        form = view.get_form(view.get_form_class())

        # Set-up expected interfaces.
        expected_interfaces = ["<Interface: Test Entity 1 (192.168.1.1)>",
                               "<Interface: Test Entity 2 (192.168.1.2)>"]

        self.assertQuerysetEqual(form.fields["source"].queryset, expected_interfaces)
        self.assertQuerysetEqual(form.fields["destination"].queryset, expected_interfaces)

    def test_initial_from_entity(self):
        """
        Tests if the choice field for interface is defaulted to first interface
        of entity passed as part of GET parameters.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?from_entity=1")
        view.object = None

        # Get the expected interface ID.
        interface = Entity.objects.get(pk=1).interface_set.all()[0]

        # Fetch the initial values.
        initial = view.get_initial()

        self.assertDictContainsSubset({"source": interface.pk}, initial)

    def test_initial_to_entity(self):
        """
        Tests if the choice field for interface is defaulted to first interface
        of entity passed as part of GET parameters.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?to_entity=1")
        view.object = None

        # Get the expected interface ID.
        interface = Entity.objects.get(pk=1).interface_set.all()[0]

        # Fetch the initial value.
        initial = view.get_initial()

        self.assertDictContainsSubset({"destination": interface.pk}, initial)

    def test_initial_invalid_from_entity(self):
        """
        Tests if the choice fields for source and destination interfaces are not
        defaulted in case invalid entity ID is passed as GET parameter.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?from_entity=10")
        view.object = None

        # Get the initial values.
        initial = view.get_initial()

        self.assertEqual(len(initial), 0)

    def test_initial_invalid_to_entity(self):
        """
        Tests if the choice fields for source and destination interfaces are not
        defaulted in case invalid entity ID is passed as GET parameter.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?to_entity=10")
        view.object = None

        # Get the initial values.
        initial = view.get_initial()

        self.assertEqual(len(initial), 0)

    def test_initial_invalid_project(self):
        """
        Tests if the choice fields for source and destination interfaces are not
        defaulted in case invalid project ID is passed as GET parameter.
        """

        # Set-up the view.
        view = CommunicationCreateView()
        view.request = RequestFactory().get("/fake-path?project=10")
        view.object = None

        # Get the initial values.
        initial = view.get_initial()

        self.assertEqual(len(initial), 0)

    def test_success_url_next(self):
        """
        Validate that the success URL is set properly after communication is
        created if "next" GET parameter is provided.
        """

        # Get the view.
        view = CommunicationCreateView.as_view()

        # Generate the request.
        source = Interface.objects.get(pk=1)
        destination = Interface.objects.get(pk=2)
        post_data = {"source": source.pk,
                     "destination": destination.pk,
                     "protocol": "TCP",
                     "port": "22",
                     "description": "SSH."}
        request = RequestFactory().post("/fake-path?next=/next-page", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request)

        self.assertEqual(response["Location"], "/next-page")
        self.assertEqual(response.status_code, 302)

    def test_success_url_no_next(self):
        """
        Validate that the success URL is set properly after communication is
        created if no "next" GET parameter is provided.
        """

        # Get the view.
        view = CommunicationCreateView.as_view()

        # Generate the request.
        source = Interface.objects.get(pk=1)
        destination = Interface.objects.get(pk=2)
        post_data = {"source": source.pk,
                     "destination": destination.pk,
                     "protocol": "TCP",
                     "port": "22",
                     "description": "SSH."}
        request = RequestFactory().post("/fake-path", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request)

        self.assertEqual(response["Location"], reverse("project", args=(1,)))
        self.assertEqual(response.status_code, 302)


class CommunicationUpdateViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = CommunicationUpdateView
    sufficient_permissions = ("change_communication",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up.
        """

        # Get the view.
        view = CommunicationUpdateView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        # Set-up expected interface.
        communication = Communication.objects.get(pk=1)

        self.assertEqual(response.context_data["communication"], communication)
        self.assertEqual(response.context_data["headline"], "Update communication Test Entity 2 -> Test Entity 1 (TCP:22)")

    def test_form_interface_limit(self):
        """
        Tests if the queryset is properly limitted to specific project's
        entity interfaces.
        """

        # Set-up the view.
        view = CommunicationUpdateView()
        view.request = RequestFactory().get("/fake-path/1")
        view.object = Communication.objects.get(pk=1)

        # Get the form.
        form = view.get_form(view.get_form_class())

        expected_interfaces = ["<Interface: Test Entity 1 (192.168.1.1)>",
                               "<Interface: Test Entity 2 (192.168.1.2)>",
                               "<Interface: Test Entity 3 (192.168.1.3)>",
                               "<Interface: Test Subnet 4 (10.10.4.0/255.255.255.0)>"]

        self.assertQuerysetEqual(form.fields["source"].queryset, expected_interfaces)
        self.assertQuerysetEqual(form.fields["destination"].queryset, expected_interfaces)

    def test_success_url_next(self):
        """
        Validate that the success URL is set properly after update if GET
        parameter is passed.
        """

        # Get the view.
        view = CommunicationUpdateView.as_view()

        # Get the communication object.
        communication = Communication.objects.get(pk=1)

        # Generate the request.
        post_data = {"source": communication.source.pk,
                     "destination": communication.destination.pk,
                     "protocol": communication.protocol,
                     "port": communication.port}
        request = RequestFactory().post("/fake-path?next=/next-page", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], "/next-page")
        self.assertEqual(response.status_code, 302)

    def test_success_url_no_next(self):
        """
        Validate that the success URL is set properly after communication is
        created if no "next" GET parameter is provided.
        """

        # Get the view.
        view = CommunicationUpdateView.as_view()

        # Get the communication object.
        communication = Communication.objects.get(pk=1)

        # Generate the request.
        post_data = {"source": communication.source.pk,
                     "destination": communication.destination.pk,
                     "protocol": communication.protocol,
                     "port": communication.port}
        request = RequestFactory().post("/fake-path/", data=post_data)
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("project", args=(communication.source.entity.project.id,)))
        self.assertEqual(response.status_code, 302)


class CommunicationDeleteViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    view_class = CommunicationDeleteView
    sufficient_permissions = ("delete_communication",)
    permission_test_view_kwargs = {"pk": 1}
    render_test_view_kwargs = {"pk": 1}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_context(self):
        """
        Verifies that the context is properly set-up when the view is called for
        specific communication.
        """

        # Get the expected entity.
        communication = Communication.objects.get(pk=1)

        # Get the view.
        view = CommunicationDeleteView.as_view()

        # Get the response.
        response = generate_get_response(view, None, pk=1)

        self.assertEqual(response.context_data["communication"], communication)
        self.assertEqual(response.context_data["headline"], "Delete communication Test Entity 2 -> Test Entity 1 (TCP:22)")

    def test_message(self):
        """
        Tests if the message gets added when the communication is deleted.
        """

        # Get the view.
        view = CommunicationDeleteView.as_view()

        # Generate the request.
        request = RequestFactory().post("/fake-path/")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertIn("Communication Test Entity 2 -> Test Entity 1 (TCP:22) has been removed.", request._messages.messages)

    def test_success_url_from_entity(self):
        """
        Validate that the success URL is set properly after delete.
        """

        # Get the view.
        view = CommunicationDeleteView.as_view()

        # Generate the request
        request = RequestFactory().post("/fake-path?from_entity=1")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(1,)))

    def test_success_url_to_entity(self):
        """
        Validate that the success URL is set properly after delete.
        """

        # Get the view.
        view = CommunicationDeleteView.as_view()

        # Generate the request
        request = RequestFactory().post("/fake-path?to_entity=1")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(1,)))

    def test_success_url_no_entity(self):
        """
        Validate that the success URL is set properly after delete.
        """

        # Get the view.
        view = CommunicationDeleteView.as_view()

        # Get the communication object.
        communication = Communication.objects.get(pk=1)

        # Generate the request
        request = RequestFactory().post("/fake-path")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request, pk=1)

        self.assertEqual(response["Location"], reverse("entity", args=(communication.source.entity.pk,)))


class ProjectDiagramTest(PermissionTestMixin, TestCase):

    view_function = staticmethod(project_diagram)
    sufficient_permissions = ("view",)
    permission_test_view_kwargs = {"pk": "1"}
    render_test_view_kwargs = {"pk": "1"}

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_invalid_project(self):
        """
        Tests if a 404 is returned if no project was found (invalid ID).
        """

        # Set-up a request.
        request = create_get_request()

        # Get the view.
        view = project_diagram

        # Validate the response.
        self.assertRaises(Http404, view, request, pk=200)

    def test_content_type(self):
        """
        Test if correct content type is being returned by the response.
        """

        # Get the view.
        view = project_diagram

        # Get the response.
        response = generate_get_response(view, pk=1)

        self.assertEqual(response['Content-Type'], "image/svg+xml")

    def test_content(self):
        """
        Tests content produced by the view.
        """

        # Get the view.
        view = project_diagram

        # Get the response.
        response = generate_get_response(view, pk=1)

        self.assertContains(response, '"-//W3C//DTD SVG 1.1//EN"')
        self.assertContains(response, "Test Project 1")


class RedirectToNextMixinTest(TestCase):

    def test_request_with_next(self):
        """
        Test if the get_success_url returns correct URL if "next" is present in
        request's GET parameters.
        """

        # Generate the request.
        request = RequestFactory().post("/fake-path?next=/next")

        # Initialise the pseudo-view.
        view = RedirectToNextMixinView(request)

        self.assertEqual("/next", view.get_success_url())

    def test_request_without_next(self):
        """
        Test if the get_success_url returns correct URL if "next" is not present
        in request's GET parameters.
        """

        # Generate the request.
        request = RequestFactory().post("/fake-path")

        # Initialise the pseudo-view.
        view = RedirectToNextMixinView(request)

        self.assertEqual("/STATIC", view.get_success_url())

    def test_request_custom_parameter_name(self):
        """
        Test if the mixin honours the custom parameter name.
        """

        # Generate the request.
        request = RequestFactory().post("/fake-path?custom=/next")

        # Initialise the pseudo-view.
        view = RedirectToNextMixinView(request)
        view.next_parameter = "custom"

        self.assertEqual("/next", view.get_success_url())


class SearchViewTest(RenderTestMixin, PermissionTestMixin, TestCase):

    sufficient_permissions = ("view",)
    view_class = SearchView

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_empty_query_error_message(self):
        """
        Verifies that an error is reported to the user in case an empty query is
        submitted.
        """

        # Get the view.
        view = SearchView.as_view()

        # Generate the request
        request = RequestFactory().get("/fake-path?q=")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True
        request._messages = FakeMessages()

        # Get the response.
        response = view(request)

        self.assertIn("Search query is not allowed to be empty.", request._messages.messages)

    def test_strip_search_term(self):
        """
        Verifies that the search term is stripped when search is performed.
        """

        # Get the view.
        view = SearchView.as_view()

        # Set-up a request.
        search_term = " \t \t something with lots of tabs \t  \t"
        stripped_search_term = "something with lots of tabs"
        request = RequestFactory().get("/fake-path?q=%s" % urlquote(search_term))
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Get the response.
        response = view(request)

        # Validate the response.
        self.assertEqual(stripped_search_term, response.context_data["search_term"])

    def test_no_query_context(self):
        """
        Tests that context is not set if no query was sent.
        """

        # Get the view.
        view = SearchView.as_view()

        # Set-up a request.
        response = generate_get_response(view)

        self.assertNotIn("entities", response.context_data)
        self.assertNotIn("projects", response.context_data)
        self.assertNotIn("search_term", response.context_data)
        # Only the "view" context variable should be present.
        self.assertEqual(1, len(response.context_data))


class APISearchViewTest(PermissionTestMixin, TestCase):

    sufficient_permissions = ("view",)
    view_class = APISearchView

    def setUp(self):
        """
        Set-up some test data.
        """

        setup_test_data()

    def test_limit_negative(self):
        """
        Test if an exception is raised in case a negative limit is requested.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Generate the request.
        request = RequestFactory().get("/fake-path?limit=-1")
        request.user = mock.Mock()
        request._dont_enforce_csrf_checks = True

        # Validate the response.
        self.assertRaisesRegexp(ValidationError, "Limit may not be a negative value.", view, request, search_term="test")

    def test_empty_query(self):
        """
        Test that the response is empty if empty query was sent.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="")

        self.assertEqual(response['Content-Type'], "application/json")
        self.assertEqual(response.content, "[]")

    def test_strip_search_term(self):
        """
        Verifies that the search term is stripped when search is performed.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="Test Entity 1")

        # Validate the response.
        expected_content = """[{"project": "Test Project 1", "url": "/conntrackt/entity/1/", "type": "entity", "name": "Test Entity 1"}]"""
        self.assertEqual(response.content, expected_content)

    def test_no_items(self):
        """
        Test the response if no items are found.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="string that does not exist")

        self.assertEqual(response['Content-Type'], "application/json")
        self.assertEqual(response.content, "[]")

    def test_entity_found(self):
        """
        Test the response if a single entity is found.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="Test Entity 1")

        expected_content = """[{"project": "Test Project 1", "url": "/conntrackt/entity/1/", "type": "entity", "name": "Test Entity 1"}]"""

        self.assertEqual(response['Content-Type'], "application/json")
        self.assertEqual(response.content, expected_content)

    def test_project_found(self):
        """
        Test the response if a single project is found.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="Test Project 1")

        expected_content = """[{"project": "Test Project 1", "url": "/conntrackt/project/1/", "type": "project", "name": "Test Project 1"}]"""

        self.assertEqual(response['Content-Type'], "application/json")
        self.assertEqual(response.content, expected_content)

    def test_multiple_items_found(self):
        """
        Test the response if multiple items are found.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="Test")

        # Verify that the JSON reply is valid.
        try:
            items = json.loads(response.content)
        except ValueError:
            self.fail("Parsing of resulting JSON has failed")

        # Verify that a list of items was returned.
        self.assertTrue(isinstance(items, list))

        # Verify each item.
        for item in items:
            # Every item must be a dictionary.
            self.assertTrue(isinstance(item, dict))
            keys = item.keys()
            # Verify that 4 specific keys are present in dictionary (project,
            #  url, name, type).
            self.assertEqual(len(keys), 4)
            self.assertIn("project", keys)
            self.assertIn("name", keys)
            self.assertIn("url", keys)
            self.assertIn("type", keys)
            # Verify the type associated with item.
            self.assertIn(item["type"], ["project", "entity"])

    def test_content_type(self):
        """
        Test if correct content type is being returned by the response.
        """

        # Get the view.
        view = APISearchView.as_view()

        # Get the response.
        response = generate_get_response(view, search_term="test")

        self.assertEqual(response['Content-Type'], "application/json")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.test import TestCase

# Application imports.
from conntrackt import iptables


class RuleTest(TestCase):
    def test_output_case(self):
        """
        Test that protocol name is lower-cased during rule generation.
        """

        rule = iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "tCp", "80", description="Web server.")
        self.assertEqual(str(rule), "-s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p tcp -m tcp --dport 80 -j ACCEPT")

    def test_output_tcp(self):
        """
        Tests that a TCP rule is generated properly.
        """

        rule = iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "TCP", "80", description="Web server.")
        self.assertEqual(str(rule), "-s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p tcp -m tcp --dport 80 -j ACCEPT")

    def test_output_udp(self):
        """
        Tests that a UDP rule is generated properly.
        """

        rule = iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "UDP", "53", description="DNS server.")
        self.assertEqual(str(rule), "-s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p udp -m udp --dport 53 -j ACCEPT")

    def test_output_icmp(self):
        """
        Tests that an ICMP rule is generated properly.
        """

        rule = iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "ICMP", "8", description="Ping.")
        self.assertEqual(str(rule), "-s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p icmp -m icmp --icmp-type 8 -j ACCEPT")

    def test_unsupported_protocol(self):
        """
        Tests that unsupported protocol will raise an exception.
        """

        self.assertRaises(ValueError, iptables.Rule, "192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255",
                          "NONEXIST", "8", description="Non-existing")


class LoopbackRuleTest(TestCase):
    def test_output(self):
        """
        Tests that a loopback rule is generated properly.
        """

        rule = iptables.LoopbackRule()
        self.assertEqual(str(rule), "-i lo -j ACCEPT")


class RelatedRuleTest(TestCase):
    def test_output(self):
        """
        Tests that a related rule is generated properly.
        """

        rule = iptables.RelatedRule()
        self.assertEqual(str(rule), "-m state --state RELATED,ESTABLISHED -j ACCEPT")


class ChainTest(TestCase):
    def test_output_empty(self):
        """
        Test generation of empty chain.
        """

        chain = iptables.Chain("INPUT", "ACCEPT")
        self.assertEqual(str(chain), ":INPUT ACCEPT [0:0]\n")

    def test_unsupported_protocol(self):
        """
        Tests that unsupported target will raise an exception.
        """

        self.assertRaises(ValueError, iptables.Chain, "INPUT", "NOTARGET")

    def test_add_rule(self):
        """
        Tests that the rule is being added to the chain properly.
        """

        chain = iptables.Chain("INPUT", "ACCEPT")
        rule = iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "TCP", "22", "SSH")
        chain.add_rule(rule)

        self.assertItemsEqual(chain.rules, [rule])

    def test_output(self):
        """
        Tests that a chain is generated properly.
        """

        chain = iptables.Chain("INPUT", "ACCEPT")

        chain.add_rule(iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "TCP", "80", "Web server"))
        chain.add_rule(iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.3/255.255.255.255", "TCP", "80", "Web server"))

        chain.add_rule(iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.2/255.255.255.255", "TCP", "22", "SSH"))
        chain.add_rule(iptables.Rule("192.168.1.1/255.255.255.255", "192.168.1.3/255.255.255.255", "TCP", "22", ""))

        expected_output = """:INPUT ACCEPT [0:0]
-A INPUT -s 192.168.1.1/255.255.255.255 -d 192.168.1.3/255.255.255.255 -p tcp -m tcp --dport 22 -j ACCEPT

# SSH
-A INPUT -s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p tcp -m tcp --dport 22 -j ACCEPT

# Web server
-A INPUT -s 192.168.1.1/255.255.255.255 -d 192.168.1.2/255.255.255.255 -p tcp -m tcp --dport 80 -j ACCEPT
-A INPUT -s 192.168.1.1/255.255.255.255 -d 192.168.1.3/255.255.255.255 -p tcp -m tcp --dport 80 -j ACCEPT

"""

        self.assertEqual(str(chain), expected_output)


class TableTest(TestCase):
    def test_output_empty(self):
        """
        Tests that an empty table is generated properly.
        """

        table = iptables.Table("filter")
        self.assertEqual(str(table), "*filter\nCOMMIT\n")

    def test_output(self):
        """
        Tests that a table is generated properly.
        """

        table = iptables.Table("filter")
        table.add_chain(iptables.Chain("INPUT", "ACCEPT"))
        table.add_chain(iptables.Chain("OUTPUT", "ACCEPT"))
        table.add_chain(iptables.Chain("FORWARD", "ACCEPT"))

        expected_output = """*filter
:INPUT ACCEPT [0:0]
:OUTPUT ACCEPT [0:0]
:FORWARD ACCEPT [0:0]
COMMIT
"""
        self.assertEqual(str(table), expected_output)

    def test_add_chain(self):
        """
        Tests that the chain is being added to the table properly.
        """

        table = iptables.Table("filter")
        chain = iptables.Chain("INPUT", "ACCEPT")
        table.add_chain(chain)

        self.assertItemsEqual(table.chains, [chain])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.conf.urls import patterns, url
from django.contrib.auth.views import login, logout

# Application imports.
from .views import IndexView, EntityView, entity_iptables, project_iptables, project_diagram
from .views import ProjectView, ProjectCreateView, ProjectUpdateView, ProjectDeleteView
from .views import LocationCreateView, LocationUpdateView, LocationDeleteView
from .views import EntityCreateView, EntityUpdateView, EntityDeleteView
from .views import InterfaceCreateView, InterfaceUpdateView, InterfaceDeleteView
from .views import CommunicationCreateView, CommunicationUpdateView, CommunicationDeleteView
from .views import SearchView, APISearchView


urlpatterns = patterns(
    'conntrackt.views',
    # Homepage/index view.
    url(r'^$', IndexView.as_view(), name="index"),

    # View for showing information about a project.
    url(r'^project/(?P<pk>\d+)/$', ProjectView.as_view(),
        name='project'),
    # View for creating a new project.
    url(r'^project/add/$', ProjectCreateView.as_view(), name="project_create"),
    # View for updating an existing project.
    url(r'^project/(?P<pk>\d+)/edit/$', ProjectUpdateView.as_view(), name="project_update"),
    # View for deleting a project.
    url(r'^project/(?P<pk>\d+)/remove/$', ProjectDeleteView.as_view(), name="project_delete"),

    # View for creating a new location.
    url(r'^location/add/$', LocationCreateView.as_view(), name="location_create"),
    # View for updating an existing location.
    url(r'^location/(?P<pk>\d+)/edit/$', LocationUpdateView.as_view(), name="location_update"),
    # View for deleting a location.
    url(r'^location/(?P<pk>\d+)/remove/$', LocationDeleteView.as_view(), name="location_delete"),

    # View for showing information about an entity.
    url(r'^entity/(?P<pk>\d+)/$', EntityView.as_view(),
        name='entity'),
    # View for creating a new entity.
    url(r'^entity/add/$', EntityCreateView.as_view(), name="entity_create"),
    # View for updating an existing entity.
    url(r'^entity/(?P<pk>\d+)/edit/$', EntityUpdateView.as_view(), name="entity_update"),
    # View for deleting an entity.
    url(r'^entity/(?P<pk>\d+)/remove/$', EntityDeleteView.as_view(), name="entity_delete"),

    # View for creating a new interface.
    url(r'^interface/add/$', InterfaceCreateView.as_view(), name="interface_create"),
    # View for updating an existing interface.
    url(r'^interface/(?P<pk>\d+)/edit/$', InterfaceUpdateView.as_view(), name="interface_update"),
    # View for deleting an interface.
    url(r'^interface/(?P<pk>\d+)/remove/$', InterfaceDeleteView.as_view(), name="interface_delete"),

    # View for creating a new communucation.
    url(r'^communication/add/$', CommunicationCreateView.as_view(), name="communication_create"),
    # View for updating an existing communication.
    url(r'^communication/(?P<pk>\d+)/edit/$', CommunicationUpdateView.as_view(), name="communication_update"),
    # View for deleting a communication.
    url(r'^communication/(?P<pk>\d+)/remove/$', CommunicationDeleteView.as_view(), name="communication_delete"),

    # View for rendering iptables rules for a specific entity.
    url(r'^entity/(?P<pk>\d+)/iptables/$', entity_iptables, name="entity_iptables"),
    # View for rendering zip file with iptables rules for all entities in a project.
    url(r'^project/(?P<project_id>\d+)/iptables/$', project_iptables, name="project_iptables"),
    # View for rendering zip file with iptables rules for all entities in a project for a specific location.
    url(r'^project/(?P<project_id>\d+)/location/(?P<location_id>\d+)/iptables/$', project_iptables, name="project_location_iptables"),

    # View for showing project communications in a diagram.
    url(r'^project/(?P<pk>\d+)/diagram/$', project_diagram, name="project_diagram"),

    # Views for logging-in/out the users.
    url(r'^login/$', login, {'template_name': 'conntrackt/login.html'}, name="login"),
    url(r'^logout/$', logout, name="logout"),

    # View for displaying the search page.
    url(r'^search/$', SearchView.as_view(), name="search"),

    # View for getting the search results in JSON format.
    url(r'^api/search/(?P<search_term>.*)$', APISearchView.as_view(), name="api_search"),
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Python third-party library imports.
import mock

# Django imports.
from django.test import TestCase
from django.db import models

# Application imports.
from conntrackt.models import SearchManager, Project


class SearchManagerTest(TestCase):

    @mock.patch("conntrackt.models.Q")
    def test_search(self, q_object):
        """
        Test the search method of custom manager.
        """

        # Create a dummy model that uses the custom manager.
        class TestModel(models.Model):
            objects = SearchManager()

        # Mock the filter call so we can check what it got called with.
        TestModel.objects.filter = mock.MagicMock()

        # Set-up an instance of mocked Q object that we'll use for validating
        # the call.
        q_instance = mock.Mock()

        # Mock the __or__ call.
        q_or_method = mock.Mock()
        q_or_method.return_value = q_instance
        q_instance.__or__ = q_or_method

        # Finaly, when the constructor is called, we should return our own
        # instance.
        q_object.return_value = q_instance

        # Call the search.
        results = TestModel.objects.search("test")

        # Check if we had bee called with our mocked Q instance.
        TestModel.objects.filter.assert_has_calls([mock.call(q_instance)])
        TestModel.objects.filter.assert_called_with(q_instance)
        q_instance.__or__.assert_called_with(q_instance)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports
from django import forms

# Application imports.
from conntrackt.forms import WidgetCSSClassFormMixin, PlaceholderFormMixin


class FormWithWidgetCSSClassFormMixin(WidgetCSSClassFormMixin, forms.Form):
    """
    Helper form for testing the WidgetCSSClassFormMixin.
    """

    field1 = forms.CharField()
    field2 = forms.CharField()


class FormWithPlaceholderFormMixin(PlaceholderFormMixin, forms.Form):

    field1 = forms.CharField()
    field2 = forms.CharField()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
INSTALL_REQUIREMENTS = [
    "django-braces~=1.12.0",
    "django-crispy-forms~=1.6.0",
    "django~=1.5.0",
    "palette~=0.2.0",
    "pydot~=1.2.0",
    "south~=1.0.0",
]

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-conntrackt',
    version='0.3.1',
    packages=find_packages(exclude=["testproject", "testproject.*"]),
    include_package_data=True,
    license='GPLv3+',
    description='A simple application for tracking connection requirements between different entities in a network.',
    long_description=README,
    url='http://projects.majic.rs/conntrackt',
    author='Branko Majic',
    author_email='branko@majic.rs',
    install_requires=INSTALL_REQUIREMENTS,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: System :: Networking :: Firewalls',
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Python standard library imports.
from types import FunctionType

# Python third-party library imports.
import mock

# Django imports.
from django.core.exceptions import PermissionDenied
from django.contrib.auth.models import User, Permission
from django.test import RequestFactory


def create_get_request(url="/fake-path/", user=None):
    """
    Helper function for generating a GET request that can be passed on to a
    view.

    Arguments:

        url - URL that should be used for the request. Default is "/fake-path/".

        user - Django user to be passed on into the request. Default is
        mock.Mock().
    """

    request = RequestFactory().get(url)

    # If user was not provided, construct one using mocking.
    if user is None:
        user = mock.Mock()

    request.user = user

    return request


def generate_get_response(view, request=None, *args, **kwargs):
    """
    Generates a get response, passing the request, positional and keyword
    arguments to it as well.

    Attributes:

        view - View function that should be called.

        request - Request object to pass into view. Default is to create a new
        request using the create_get_request() call.

        *args - Additional positional arguments that will be passed into view.

        *kwargs - Additional keyword arguments that will be passed into view.
    """

    # If no request was provided, construct it.
    if request is None:
        request = create_get_request()

    return view(request, *args, **kwargs)


class PermissionTestMixin(object):
    """
    Mixin class for testing if permission requirement is applied properly for
    accessing a view.

    In order to use this mixin, add it the left side of the class list the test
    is inheriting from, and configure it providing the following class options:

        view_class - Class used for the CBV that will be tested.
        view_function - View function that will be tested.
        sufficient_permissions - Permissions sufficient to gain access to view.
        permission_test_view_args - Positional arguments to pass to the view.
        permission_test_view_kwargs - Keyword arguments to pass to the view.
    """

    view_class = None
    view_function = None
    permission_test_view_args = ()
    permission_test_view_kwargs = {}
    sufficient_permissions = ()

    def __init__(self, *args, **kwargs):
        """
        Initialises the mixin. Takes care of some basic validation of passed
        configuration options.
        """

        super(PermissionTestMixin, self).__init__(*args, **kwargs)

        if self.view_class is None and self.view_function is None:
            raise ValueError("Permission test mixin configured improperly - no CBV class or function was supplied via parameters 'view_class' or 'view_function'.")

        if self.view_function is not None and type(self.view_function) is not FunctionType:
            raise ValueError("Permission test mixin configured improperly - provided 'view_function' is not function. Did you forget to wrap the function with staticmethod() perhaps?")

        if type(self.permission_test_view_kwargs) is not dict:
            raise ValueError("Permission text mixin configured improperly - parameter 'permission_test_view_kwargs' must be a dictionary.")

        if type(self.permission_test_view_args) is not tuple:
            raise ValueError("Permission text mixin configured improperly - parameter 'permission_test_view_args' must be a tuple.")

        if type(self.sufficient_permissions) is not tuple:
            raise ValueError("Permission text mixin configured improperly - parameter 'sufficient_permissions' must be a tuple.")

    def test_permission_granted(self):
        # Set-up a request from user with sufficient privileges.
        request = RequestFactory().get("/fake-path")
        user = User.objects.create(username="user", password="password")
        for permission in self.sufficient_permissions:
            user.user_permissions.add(Permission.objects.get(codename=permission))
        request.user = user

        # Get the view.
        if self.view_class is not None:
            view = self.view_class.as_view()
        elif self.view_function is not None:
            view = self.view_function

        # Verify that permission is granted
        args = self.permission_test_view_args
        kwargs = self.permission_test_view_kwargs

        try:
            response = view(request, *args, **kwargs)
        except PermissionDenied:
            self.fail("Failed to access view with user privileges: %s" % str(self.sufficient_permissions))

        self.assertEqual(response.status_code, 200)

    def test_permission_denied(self):
        # Set-up a request from user with insufficient privileges.
        request = RequestFactory().get("/fake-path")
        request.user = User.objects.create(username="user", password="password")

        # Get the view.
        if self.view_class:
            view = self.view_class.as_view()
        elif self.view_function:
            view = self.view_function

        # Verify that permission is denied.
        args = self.permission_test_view_args
        kwargs = self.permission_test_view_kwargs
        self.assertRaises(PermissionDenied, view, request, *args, **kwargs)


class FakeMessages(object):
    """
    Helper class for mocking the Django messages framework.
    """

    def __init__(self):
        """
        Initalises the message framework mocker.

        Set-ups the messages list prpoperty.
        """
        self.messages = []

    def add(self, level, message, extra_tags):
        """
        Adds a message.
        """

        self.messages.append(message)


class RenderTestMixin(object):
    """
    Mixin class for testing that response rendering works correctly.

    In order to use this mixin, add it the left side of the class list the test
    is inheriting from, and configure it providing the following class options:

    view_class - Class used for the CBV that will be tested.
    view_function - View function that will be tested.
    render_test_view_args - Positional arguments to pass to the view.
    render_test_view_kwargs - Keyword arguments to pass to the view.
    """

    view_class = None
    view_function = None
    render_test_view_args = ()
    render_test_view_kwargs = {}

    def __init__(self, *args, **kwargs):
        """
        Initialises the mixin. Takes care of some basic validation of
        passed configuration options.
        """

        super(RenderTestMixin, self).__init__(*args, **kwargs)

        if self.view_class is None and self.view_function is None:
            raise ValueError("Render test mixin configured improperly - no CBV class or function was supplied via parameters 'view_class' or 'view_function'.")

        if type(self.render_test_view_kwargs) is not dict:
            raise ValueError("Render test mixin configured improperly - parameter 'render_test_view_kwargs' must be a dictionary.")

        if type(self.render_test_view_args) is not tuple:
            raise ValueError("Render test mixin configured improperly - parameter 'render_test_view_args' must be a tuple.")

    def test_no_render_exception_thrown_on_GET(self):

        # Get the view.
        if self.view_class is not None:
            view = self.view_class.as_view()
        elif self.view_function is not None:
            view = self.view_function

        # Grab the response.
        args = self.render_test_view_args
        kwargs = self.render_test_view_kwargs
        response = generate_get_response(view, *args, **kwargs)

        # Should not raise.
        response.render()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django import template
from django.core import urlresolvers


# Get an instance of Django's template library.
register = template.Library()


@register.simple_tag(takes_context=False)
def html_link(text, view, *args, **kwargs):
    """
    A small wrapper for showing HTML links.

    Positional arguments:

        text - Text that should be used as a link.

        view - View name for which the URL should be shown

        args - Additional positional arguments that will be passed to resolver
        for creating the URL.

    Keyword arguments:

        id - Identifier for the <a> HTML element.

        class - Class(es) for the <a> HTML element.

        title - Title for the HTML <a> element.

        get - Additional GET parameter that should be appended to the URL.

    """

    # Generate the URL by using the supplied view name and arguments that should
    # be passed to the view.
    url = urlresolvers.reverse(view, args=args)

    # Set-up the base pattern (url, parameters, text).
    pattern = '<a href="%s" %s>%s</a>'

    # Iterate over keyword arguments, and if they're supported, add them to
    # parameters.
    params = ""
    for key, value in kwargs.iteritems():
        if key in ("class", "title", "id"):
            params += '%s="%s" ' % (key, value)
        elif key == "get":
            url += "?%s" % value
        else:
            raise template.TemplateSyntaxError("Unknown argument for 'advhtml_link' tag: %r" % key)

    # Render the tag.
    return pattern % (url, params, text)


@register.simple_tag(takes_context=True)
def active_link(context, url_name, return_value='active', **kwargs):
    """
    This template tag can be used to check if the provided URL matches against
    the path from the request or not.

    Arguments:

      context - Context of the current view being called.

      url_name - Name of the URL that's being checked against current path from
      request.
    """

    matches = current_url_equals(context, url_name, **kwargs)

    return return_value if matches else ''


def current_url_equals(context, url_name, **kwargs):
    """
    Helper function for checking if the specified URL corresponds to the current
    request path in the context.

    Arguments:

      - context - Context of the view being rendered.

      - url_name - Name of the URL against which the context request path is
      being checked.
    """

    # Assume that we have not been able to resolve the request path to an URL.
    resolved = False
    try:
        # Use the request path, and resolve it to a URL name.
        resolved = urlresolvers.resolve(context.get('request').path)
    except urlresolvers.Resolver404:
        # This means we haven't been able to resolve the path from request.
        pass

    # If the request was resolved and URL names match, verify that the kwargs
    # match as well.
    matches = resolved and resolved.url_name == url_name
    if matches and kwargs:
        for key in kwargs:
            kwarg = kwargs.get(key)
            resolved_kwarg = resolved.kwargs.get(key)
            if not resolved_kwarg or kwarg != resolved_kwarg:
                return False

    return matches



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Third-party application imports.
import factory

# Application imports.
from conntrackt.models import Project, Location, Entity, Interface, Communication


class ProjectFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing projects where name is set to "Test Project N", and N
    is an increasing sequence. Description is based on name.
    """

    FACTORY_FOR = Project

    @factory.sequence
    def name(n):
        return "Test Project %d" % n

    description = factory.LazyAttribute(lambda d: "This is %s." % d.name)


class LocationFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing locations where name is set to "Test Location N", and
    N is an increasing sequence. Description is based on name.
    """

    FACTORY_FOR = Location

    @factory.sequence
    def name(n):
        return "Test Location %d" % n

    description = factory.LazyAttribute(lambda d: "This is %s." % d.name)


class ServerInterfaceFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing server interfaces. Interface IP address is set to
    192.168.1.N, where N is a sequence. The netmask is always set to
    "255.255.255.255". The interface name is always set to eth0.
    """

    FACTORY_FOR = Interface
    netmask = "255.255.255.255"
    name = "eth0"

    @factory.sequence
    def address(n):
        return "192.168.1.%d" % (n)


class SubnetInterfaceFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing subnet "interfaces". Interface IP address is set to
    10.10.N.0, where N is a sequence. Interface name is always set to
    switch0. Netmask is set to "255.255.255.0".
    """

    FACTORY_FOR = Interface
    netmask = "255.255.255.0"
    name = "switch0"

    @factory.sequence
    def address(n):
        return "10.10.%d.0" % (n)


class ServerEntityFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing server entities where name is set to "Test Entity N",
    and N is an increasing sequence.
    """

    FACTORY_FOR = Entity

    interface = factory.RelatedFactory(ServerInterfaceFactory, "entity")

    @factory.sequence
    def name(n):
        return "Test Entity %d" % n

    description = factory.LazyAttribute(lambda d: "This is %s." % d.name)


class SubnetEntityFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing subnet entities where name is set to "Test Subnet N",
    and N is an increasing sequence.
    """

    FACTORY_FOR = Entity

    interface = factory.RelatedFactory(SubnetInterfaceFactory, "entity")

    @factory.sequence
    def name(n):
        return "Test Subnet %d" % n

    description = factory.LazyAttribute(lambda d: "This is %s." % d.name)


class CommunicationFactory(factory.django.DjangoModelFactory):
    """
    Factory for producing communications. The descriptin of communication will
    be based on protocol and port.
    """

    FACTORY_FOR = Communication

    description = factory.LazyAttribute(lambda d: "Communicate over %s:%s" % (d.protocol, d.port))


def setup_test_data():
    """
    Sets-up some test data for testing more complex functionality.
    """

    for factory in ProjectFactory, LocationFactory, ServerEntityFactory, SubnetEntityFactory, ServerInterfaceFactory, SubnetInterfaceFactory, CommunicationFactory:
        factory.reset_sequence()

    project1 = ProjectFactory(pk=1)
    project2 = ProjectFactory(pk=2)

    location1 = LocationFactory(pk=1)
    location2 = LocationFactory(pk=2)

    entity1 = ServerEntityFactory(pk=1, project=project1, location=location1)
    entity2 = ServerEntityFactory(pk=2, project=project1, location=location1)
    entity3 = ServerEntityFactory(pk=3, project=project1, location=location2)
    entity4 = SubnetEntityFactory(pk=4, project=project1, location=location2)
    entity5 = ServerEntityFactory(pk=5, project=project2, location=location1)

    communication1 = CommunicationFactory(pk=1, source_id=2, destination_id=1, protocol="TCP", port="22")
    communication2 = CommunicationFactory(pk=2, source_id=2, destination_id=1, protocol="ICMP", port="8")
    communication3 = CommunicationFactory(pk=3, source_id=3, destination_id=1, protocol="TCP", port="3306")
    communication4 = CommunicationFactory(pk=4, source_id=1, destination_id=3, protocol="UDP", port="53")
    communication5 = CommunicationFactory(pk=5, source_id=4, destination_id=1, protocol="TCP", port="22")
    communication6 = CommunicationFactory(pk=6, source_id=1, destination_id=2, protocol="UDP", port="123")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# Django imports.
from django.test import TestCase

# Python third-party library imports.
import mock

# Application imports.
from conntrackt.forms import WidgetCSSClassFormMixin, PlaceholderFormMixin
from conntrackt.forms import EntityForm, InterfaceForm, CommunicationForm

# Test imports.
from .forms import FormWithWidgetCSSClassFormMixin, FormWithPlaceholderFormMixin


class WidgetCSSClassFormMixinTest(TestCase):
    """
    Tests the form mixin.
    """

    def test_apply_to_all(self):
        """
        Test if CSS class is appled to all form field widgets.
        """

        # Set-up the form.
        form_class = FormWithWidgetCSSClassFormMixin
        form_class.widget_css_classes = {"ALL": "test"}
        form = form_class()

        self.assertEqual(form.fields["field1"].widget.attrs["class"], "test")
        self.assertEqual(form.fields["field2"].widget.attrs["class"], "test")

    def test_apply_to_single(self):
        """
        Test if CSS class is appled to a single field widget.
        """

        # Set-up the form.
        form_class = FormWithWidgetCSSClassFormMixin
        form_class.widget_css_classes = {"field2": "test"}
        form = form_class()

        self.assertEqual(form.fields["field1"].widget.attrs.get("class", None), None)
        self.assertEqual(form.fields["field2"].widget.attrs["class"], "test")

    def test_apply_multiple(self):
        """
        Tests if different class is applied to multiple form field widgets.
        """

        # Set-up the form.
        form_class = FormWithWidgetCSSClassFormMixin
        form_class.widget_css_classes = {"field1": "f1",
                                         "field2": "f2"}
        form = form_class()

        self.assertEqual(form.fields["field1"].widget.attrs["class"], "f1")
        self.assertEqual(form.fields["field2"].widget.attrs["class"], "f2")

    def test_apply_to_all_additional(self):
        """
        Tests if all widgets get the same CSS class in addition to induvidual
        ones.
        """

        # Set-up the form.
        form_class = FormWithWidgetCSSClassFormMixin
        form_class.widget_css_classes = {"field1": "f1",
                                         "field2": "f2",
                                         "ALL": "all"}
        form = form_class()

        self.assertEqual(sorted(["f1", "all"]), sorted(form.fields["field1"].widget.attrs["class"].split(" ")))
        self.assertEqual(sorted(["f2", "all"]), sorted(form.fields["field2"].widget.attrs["class"].split(" ")))


class PlaceholderFormMixinTest(TestCase):
    """
    Tests the form mixin.
    """

    def test_apply_one(self):
        """
        Test if a single placeholder is applied to a form field widget.
        """

        # Set-up the form.
        form_class = FormWithPlaceholderFormMixin
        form_class.widget_placeholders = {"field1": "place1"}
        form = form_class()

        self.assertEqual(form.fields["field1"].widget.attrs["placeholder"], "place1")
        self.assertEqual(form.fields["field2"].widget.attrs.get("placeholder", None), None)

    def test_apply_multiple(self):
        """
        Test if multiple placeholders are applied to form field widgets.
        """

        # Set-up the form.
        form_class = FormWithPlaceholderFormMixin
        form_class.widget_placeholders = {"field1": "place1",
                                   "field2": "place2"}
        form = form_class()

        self.assertEqual(form.fields["field1"].widget.attrs["placeholder"], "place1")
        self.assertEqual(form.fields["field2"].widget.attrs["placeholder"], "place2")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 Branko Majic
#
# This file is part of Django Conntrackt.
#
# Django Conntrackt is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# Django Conntrackt is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# Django Conntrackt.  If not, see <http://www.gnu.org/licenses/>.
#


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding unique constraint on 'Interface', fields ['netmask', 'address', 'entity']
        db.create_unique(u'conntrackt_interface', ['netmask', 'address', 'entity_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'Interface', fields ['netmask', 'address', 'entity']
        db.delete_unique(u'conntrackt_interface', ['netmask', 'address', 'entity_id'])


    models = {
        u'conntrackt.communication': {
            'Meta': {'unique_together': "(('source', 'destination', 'protocol', 'port'),)", 'object_name': 'Communication'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'destination': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'destination_set'", 'to': u"orm['conntrackt.Interface']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'port': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'protocol': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'source_set'", 'to': u"orm['conntrackt.Interface']"})
        },
        u'conntrackt.entity': {
            'Meta': {'unique_together': "(('name', 'project'),)", 'object_name': 'Entity'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'location': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['conntrackt.Location']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['conntrackt.Project']"})
        },
        u'conntrackt.interface': {
            'Meta': {'unique_together': "(('name', 'entity'), ('entity', 'address', 'netmask'))", 'object_name': 'Interface'},
            'address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "'Main network interface.'", 'blank': 'True'}),
            'entity': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['conntrackt.Entity']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "'eth0'", 'max_length': '100'}),
            'netmask': ('django.db.models.fields.IPAddressField', [], {'default': "'255.255.255.255'", 'max_length': '15'})
        },
        u'conntrackt.location': {
            'Meta': {'object_name': 'Location'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        },
        u'conntrackt.project': {
            'Meta': {'object_name': 'Project'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'})
        }
    }

    complete_apps = ['conntrackt']